#include <cstdlib>
#include <iostream>
#include <cstring> // per memcpy
#include <algorithm> // per copy
#include <iterator> // per ostream_iterator
using namespace std;


template < int N >  // non-type parameters (solo tipi interi)
class Noise {
  char * mem_;
  // helper function che fa deep copy
  void copy_(const Noise & n) {
    memcpy( mem_, n.mem_, N*sizeof(char) );
  }
public:
  Noise() {
    mem_ = new char[N];
    for (int i = 0; i<N; ++i) mem_[i] = rand(); // attnz: cast!
  }
  ~Noise() {
    delete [] mem_;
  }
  
  // DEEP COPY (viene copiata tutta la memoria puntata da mem_)
  Noise(const Noise & n) {
    mem_ = new char[N];
    copy_(n);
  }
  Noise & operator = (const Noise & n) {
    copy_(n);
    return *this;
  }

  // MOVE (viene copiato solo il puntatore, e invalidato l'oggetto sorgente)
  void move(Noise & n) {
    delete [] mem_;
    mem_ = n.mem_;
    n.mem_ = NULL;
  }

  // SWAP (volendo c'e' un template nella STL)
  void swap(Noise & n) {
    char * tmp = mem_;
    mem_ = n.mem_;
    n.mem_ = tmp;
  }

  void print(ostream & os) {
    if (mem_)
      // qui uso due puntatori come iteratori random_access
      copy(mem_,mem_+N,ostream_iterator<int>(os," "));
    else
      os << "Oggetto non valido";
    os << endl;
  }
};


int main() {

  Noise<6> A; Noise<6> B; // nota: con interi diversi sarebbero TIPI diversi

  cout << endl << "A: "; A.print(cout);
  cout << "B: "; B.print(cout);
  cout << endl;

  cout << "--> move di B in A <--" << endl; A.move(B);

  cout << endl << "A: "; A.print(cout);
  cout << "B: "; B.print(cout);
  cout << endl;
}
